// $Id: wyckoff.cpp,v 1.11 2001/11/26 23:05:23 rwgk Exp $
/* Copyright (c) 2001 The Regents of the University of California through
   E.O. Lawrence Berkeley National Laboratory, subject to approval by the
   U.S. Department of Energy. See files COPYRIGHT.txt and
   cctbx/LICENSE.txt for further details.

   Revision history:
     2001 Oct 12: SpecialPosition -> SiteSymmetry (R.W. Grosse-Kunstleve)
     2001 Sep 13: SpaceGroupType -> SpaceGroupInfo (R.W. Grosse-Kunstleve)
     2001 Jul 02: Merged from CVS branch sgtbx_special_pos (rwgk)
     Created: 17-Jun-2001 (R.W. Grosse-Kunstleve)
 */

#include <cctbx/sgtbx/coordinates.h>
#include <cctbx/basic/define_range.h>

namespace cctbx { namespace sgtbx {
  namespace tables {
    namespace ReferenceSettings {

      // Based on ~rwgk/spgr/altwyk/altwyk-9706061705:
      //   python reference-altwyk.py > reference-only-altwyk-9706061705
      //   cform-std-only.pl reference-only-altwyk-9706061705
      // Manually modified for C++.
      // Reprocessed with sgtbx to obtain SpecialOp that correctly
      // average. SpecialOp are chosen such that SpecialOp * X = X
      // when all elements of X are in the interval )-0.5,0.5].

      struct RawWyckoffPosition {
        int M;
        const char *xyz;
      };

      struct RawWyckoffTable {
        int N;
        const RawWyckoffPosition *RWO;
      };

// BEGIN_COMPILED_IN_REFERENCE_DATA
      static const RawWyckoffPosition W2[] = {
      { 1, "1/2,1/2,1/2"},
      { 1, "0,1/2,1/2"},
      { 1, "1/2,0,1/2"},
      { 1, "1/2,1/2,0"},
      { 1, "1/2,0,0"},
      { 1, "0,1/2,0"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W3[] = {
      { 1, "1/2,y,1/2"},
      { 1, "1/2,y,0"},
      { 1, "0,y,1/2"},
      { 1, "0,y,0"}
      };
      static const RawWyckoffPosition W5[] = {
      { 2, "0,y,1/2"},
      { 2, "0,y,0"}
      };
      static const RawWyckoffPosition W6[] = {
      { 1, "x,1/2,z"},
      { 1, "x,0,z"}
      };
      static const RawWyckoffPosition W8[] = {
      { 2, "x,0,z"}
      };
      static const RawWyckoffPosition W10[] = {
      { 2, "x,1/2,z"},
      { 2, "x,0,z"},
      { 2, "1/2,y,1/2"},
      { 2, "0,y,1/2"},
      { 2, "1/2,y,0"},
      { 2, "0,y,0"},
      { 1, "1/2,1/2,1/2"},
      { 1, "1/2,0,1/2"},
      { 1, "0,1/2,1/2"},
      { 1, "1/2,1/2,0"},
      { 1, "1/2,0,0"},
      { 1, "0,0,1/2"},
      { 1, "0,1/2,0"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W11[] = {
      { 2, "x,1/4,z"},
      { 2, "1/2,0,1/2"},
      { 2, "0,0,1/2"},
      { 2, "1/2,0,0"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W12[] = {
      { 4, "x,0,z"},
      { 4, "0,y,1/2"},
      { 4, "0,y,0"},
      { 4, "1/4,1/4,1/2"},
      { 4, "1/4,1/4,0"},
      { 2, "0,1/2,1/2"},
      { 2, "0,0,1/2"},
      { 2, "0,1/2,0"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W13[] = {
      { 2, "1/2,y,1/4"},
      { 2, "0,y,1/4"},
      { 2, "1/2,0,0"},
      { 2, "0,1/2,0"},
      { 2, "1/2,1/2,0"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W14[] = {
      { 2, "1/2,0,1/2"},
      { 2, "0,0,1/2"},
      { 2, "1/2,0,0"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W15[] = {
      { 4, "0,y,1/4"},
      { 4, "1/4,1/4,1/2"},
      { 4, "1/4,1/4,0"},
      { 4, "0,1/2,0"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W16[] = {
      { 2, "1/2,1/2,z"},
      { 2, "0,1/2,z"},
      { 2, "1/2,0,z"},
      { 2, "0,0,z"},
      { 2, "1/2,y,1/2"},
      { 2, "1/2,y,0"},
      { 2, "0,y,1/2"},
      { 2, "0,y,0"},
      { 2, "x,1/2,1/2"},
      { 2, "x,1/2,0"},
      { 2, "x,0,1/2"},
      { 2, "x,0,0"},
      { 1, "1/2,1/2,1/2"},
      { 1, "0,1/2,1/2"},
      { 1, "1/2,0,1/2"},
      { 1, "1/2,1/2,0"},
      { 1, "0,0,1/2"},
      { 1, "0,1/2,0"},
      { 1, "1/2,0,0"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W17[] = {
      { 2, "1/2,y,1/4"},
      { 2, "0,y,1/4"},
      { 2, "x,1/2,0"},
      { 2, "x,0,0"}
      };
      static const RawWyckoffPosition W18[] = {
      { 2, "0,1/2,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W20[] = {
      { 4, "0,y,1/4"},
      { 4, "x,0,0"}
      };
      static const RawWyckoffPosition W21[] = {
      { 4, "1/4,1/4,z"},
      { 4, "0,1/2,z"},
      { 4, "0,0,z"},
      { 4, "0,y,1/2"},
      { 4, "0,y,0"},
      { 4, "x,0,1/2"},
      { 4, "x,0,0"},
      { 2, "0,0,1/2"},
      { 2, "1/2,0,1/2"},
      { 2, "0,1/2,0"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W22[] = {
      { 8, "x,1/4,1/4"},
      { 8, "1/4,y,1/4"},
      { 8, "1/4,1/4,z"},
      { 8, "0,0,z"},
      { 8, "0,y,0"},
      { 8, "x,0,0"},
      { 4, "1/4,1/4,-1/4"},
      { 4, "1/4,1/4,1/4"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W23[] = {
      { 4, "0,1/2,z"},
      { 4, "0,0,z"},
      { 4, "1/2,y,0"},
      { 4, "0,y,0"},
      { 4, "x,0,1/2"},
      { 4, "x,0,0"},
      { 2, "0,1/2,0"},
      { 2, "0,0,1/2"},
      { 2, "1/2,0,0"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W24[] = {
      { 4, "0,1/4,z"},
      { 4, "1/4,y,0"},
      { 4, "x,0,1/4"}
      };
      static const RawWyckoffPosition W25[] = {
      { 2, "1/2,y,z"},
      { 2, "0,y,z"},
      { 2, "x,1/2,z"},
      { 2, "x,0,z"},
      { 1, "1/2,1/2,z"},
      { 1, "1/2,0,z"},
      { 1, "0,1/2,z"},
      { 1, "0,0,z"}
      };
      static const RawWyckoffPosition W26[] = {
      { 2, "1/2,y,z"},
      { 2, "0,y,z"}
      };
      static const RawWyckoffPosition W27[] = {
      { 2, "1/2,1/2,z"},
      { 2, "1/2,0,z"},
      { 2, "0,1/2,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W28[] = {
      { 2, "1/4,y,z"},
      { 2, "0,1/2,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W30[] = {
      { 2, "1/2,0,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W31[] = {
      { 2, "0,y,z"}
      };
      static const RawWyckoffPosition W35[] = {
      { 4, "0,y,z"},
      { 4, "x,0,z"},
      { 4, "1/4,1/4,z"},
      { 2, "0,1/2,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W36[] = {
      { 4, "0,y,z"}
      };
      static const RawWyckoffPosition W37[] = {
      { 4, "1/4,1/4,z"},
      { 4, "0,1/2,z"},
      { 4, "0,0,z"}
      };
      static const RawWyckoffPosition W38[] = {
      { 4, "1/2,y,z"},
      { 4, "0,y,z"},
      { 4, "x,0,z"},
      { 2, "1/2,0,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W39[] = {
      { 4, "x,1/4,z"},
      { 4, "1/2,0,z"},
      { 4, "0,0,z"}
      };
      static const RawWyckoffPosition W40[] = {
      { 4, "1/4,y,z"},
      { 4, "0,0,z"}
      };
      static const RawWyckoffPosition W41[] = {
      { 4, "0,0,z"}
      };
      static const RawWyckoffPosition W42[] = {
      { 8, "x,0,z"},
      { 8, "0,y,z"},
      { 8, "1/4,1/4,z"},
      { 4, "0,0,z"}
      };
      static const RawWyckoffPosition W43[] = {
      { 8, "0,0,z"}
      };
      static const RawWyckoffPosition W44[] = {
      { 4, "0,y,z"},
      { 4, "x,0,z"},
      { 2, "0,1/2,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W45[] = {
      { 4, "0,1/2,z"},
      { 4, "0,0,z"}
      };
      static const RawWyckoffPosition W47[] = {
      { 4, "x,y,1/2"},
      { 4, "x,y,0"},
      { 4, "x,1/2,z"},
      { 4, "x,0,z"},
      { 4, "1/2,y,z"},
      { 4, "0,y,z"},
      { 2, "1/2,1/2,z"},
      { 2, "1/2,0,z"},
      { 2, "0,1/2,z"},
      { 2, "0,0,z"},
      { 2, "1/2,y,1/2"},
      { 2, "1/2,y,0"},
      { 2, "0,y,1/2"},
      { 2, "0,y,0"},
      { 2, "x,1/2,1/2"},
      { 2, "x,1/2,0"},
      { 2, "x,0,1/2"},
      { 2, "x,0,0"},
      { 1, "1/2,1/2,1/2"},
      { 1, "0,1/2,1/2"},
      { 1, "1/2,1/2,0"},
      { 1, "0,1/2,0"},
      { 1, "1/2,0,1/2"},
      { 1, "0,0,1/2"},
      { 1, "1/2,0,0"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W48[] = {
      { 4, "1/4,-1/4,z"},
      { 4, "1/4,1/4,z"},
      { 4, "-1/4,y,1/4"},
      { 4, "1/4,y,1/4"},
      { 4, "x,1/4,-1/4"},
      { 4, "x,1/4,1/4"},
      { 4, "0,0,0"},
      { 4, "1/2,1/2,1/2"},
      { 2, "1/4,-1/4,1/4"},
      { 2, "1/4,1/4,-1/4"},
      { 2, "-1/4,1/4,1/4"},
      { 2, "1/4,1/4,1/4"}
      };
      static const RawWyckoffPosition W49[] = {
      { 4, "x,y,0"},
      { 4, "1/2,0,z"},
      { 4, "0,1/2,z"},
      { 4, "1/2,1/2,z"},
      { 4, "0,0,z"},
      { 4, "1/2,y,1/4"},
      { 4, "0,y,1/4"},
      { 4, "x,1/2,1/4"},
      { 4, "x,0,1/4"},
      { 2, "1/2,1/2,1/4"},
      { 2, "0,1/2,1/4"},
      { 2, "1/2,0,1/4"},
      { 2, "0,0,1/4"},
      { 2, "1/2,0,0"},
      { 2, "0,1/2,0"},
      { 2, "1/2,1/2,0"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W50[] = {
      { 4, "1/4,-1/4,z"},
      { 4, "1/4,1/4,z"},
      { 4, "1/4,y,1/2"},
      { 4, "1/4,y,0"},
      { 4, "x,1/4,1/2"},
      { 4, "x,1/4,0"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"},
      { 2, "1/4,1/4,1/2"},
      { 2, "-1/4,1/4,1/2"},
      { 2, "-1/4,1/4,0"},
      { 2, "1/4,1/4,0"}
      };
      static const RawWyckoffPosition W51[] = {
      { 4, "1/4,y,z"},
      { 4, "x,1/2,z"},
      { 4, "x,0,z"},
      { 4, "0,y,1/2"},
      { 4, "0,y,0"},
      { 2, "1/4,1/2,z"},
      { 2, "1/4,0,z"},
      { 2, "0,1/2,1/2"},
      { 2, "0,0,1/2"},
      { 2, "0,1/2,0"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W52[] = {
      { 4, "x,1/4,1/4"},
      { 4, "1/4,0,z"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W53[] = {
      { 4, "0,y,z"},
      { 4, "1/4,y,1/4"},
      { 4, "x,1/2,0"},
      { 4, "x,0,0"},
      { 2, "0,1/2,0"},
      { 2, "1/2,1/2,0"},
      { 2, "1/2,0,0"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W54[] = {
      { 4, "1/4,1/2,z"},
      { 4, "1/4,0,z"},
      { 4, "0,y,1/4"},
      { 4, "0,1/2,0"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W55[] = {
      { 4, "x,y,1/2"},
      { 4, "x,y,0"},
      { 4, "0,1/2,z"},
      { 4, "0,0,z"},
      { 2, "0,1/2,1/2"},
      { 2, "0,1/2,0"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W56[] = {
      { 4, "1/4,-1/4,z"},
      { 4, "1/4,1/4,z"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W57[] = {
      { 4, "x,y,1/4"},
      { 4, "x,1/4,0"},
      { 4, "1/2,0,0"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W58[] = {
      { 4, "x,y,0"},
      { 4, "0,1/2,z"},
      { 4, "0,0,z"},
      { 2, "0,1/2,1/2"},
      { 2, "0,1/2,0"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W59[] = {
      { 4, "x,1/4,z"},
      { 4, "1/4,y,z"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"},
      { 2, "1/4,-1/4,z"},
      { 2, "1/4,1/4,z"}
      };
      static const RawWyckoffPosition W60[] = {
      { 4, "0,y,1/4"},
      { 4, "0,1/2,0"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W61[] = {
      { 4, "0,0,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W62[] = {
      { 4, "x,1/4,z"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W63[] = {
      { 8, "x,y,1/4"},
      { 8, "0,y,z"},
      { 8, "x,0,0"},
      { 8, "1/4,1/4,0"},
      { 4, "0,y,1/4"},
      { 4, "0,1/2,0"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W64[] = {
      { 8, "0,y,z"},
      { 8, "1/4,y,1/4"},
      { 8, "x,0,0"},
      { 8, "1/4,1/4,0"},
      { 4, "1/2,0,0"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W65[] = {
      { 8, "x,y,1/2"},
      { 8, "x,y,0"},
      { 8, "x,0,z"},
      { 8, "0,y,z"},
      { 8, "1/4,1/4,z"},
      { 4, "0,1/2,z"},
      { 4, "0,0,z"},
      { 4, "0,y,1/2"},
      { 4, "0,y,0"},
      { 4, "x,0,1/2"},
      { 4, "x,0,0"},
      { 4, "1/4,1/4,1/2"},
      { 4, "1/4,1/4,0"},
      { 2, "0,0,1/2"},
      { 2, "1/2,0,1/2"},
      { 2, "1/2,0,0"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W66[] = {
      { 8, "x,y,0"},
      { 8, "1/4,1/4,z"},
      { 8, "0,1/2,z"},
      { 8, "0,0,z"},
      { 8, "0,y,1/4"},
      { 8, "x,0,1/4"},
      { 4, "1/4,-1/4,0"},
      { 4, "1/4,1/4,0"},
      { 4, "0,1/2,0"},
      { 4, "0,0,0"},
      { 4, "0,1/2,1/4"},
      { 4, "0,0,1/4"}
      };
      static const RawWyckoffPosition W67[] = {
      { 8, "x,1/4,z"},
      { 8, "0,y,z"},
      { 8, "1/4,0,z"},
      { 8, "1/4,y,1/2"},
      { 8, "1/4,y,0"},
      { 8, "x,0,1/2"},
      { 8, "x,0,0"},
      { 4, "0,1/4,z"},
      { 4, "1/4,1/4,1/2"},
      { 4, "1/4,1/4,0"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"},
      { 4, "1/4,0,1/2"},
      { 4, "1/4,0,0"}
      };
      static const RawWyckoffPosition W68[] = {
      { 8, "1/4,0,z"},
      { 8, "0,1/4,z"},
      { 8, "0,y,1/4"},
      { 8, "x,1/4,1/4"},
      { 8, "0,0,0"},
      { 8, "1/4,-1/4,0"},
      { 4, "0,1/4,-1/4"},
      { 4, "0,1/4,1/4"}
      };
      static const RawWyckoffPosition W69[] = {
      {16, "x,y,0"},
      {16, "x,0,z"},
      {16, "0,y,z"},
      {16, "x,1/4,1/4"},
      {16, "1/4,y,1/4"},
      {16, "1/4,1/4,z"},
      { 8, "0,0,z"},
      { 8, "0,y,0"},
      { 8, "x,0,0"},
      { 8, "1/4,1/4,1/4"},
      { 8, "1/4,1/4,0"},
      { 8, "1/4,0,1/4"},
      { 8, "0,1/4,1/4"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W70[] = {
      {16, "1/8,1/8,z"},
      {16, "1/8,y,1/8"},
      {16, "x,1/8,1/8"},
      {16, "1/2,1/2,1/2"},
      {16, "0,0,0"},
      { 8, "1/8,1/8,-3/8"},
      { 8, "1/8,1/8,1/8"}
      };
      static const RawWyckoffPosition W71[] = {
      { 8, "x,y,0"},
      { 8, "x,0,z"},
      { 8, "0,y,z"},
      { 8, "1/4,1/4,1/4"},
      { 4, "1/2,0,z"},
      { 4, "0,0,z"},
      { 4, "0,y,1/2"},
      { 4, "0,y,0"},
      { 4, "x,1/2,0"},
      { 4, "x,0,0"},
      { 2, "1/2,0,1/2"},
      { 2, "1/2,1/2,0"},
      { 2, "0,1/2,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W72[] = {
      { 8, "x,y,0"},
      { 8, "0,1/2,z"},
      { 8, "0,0,z"},
      { 8, "0,y,1/4"},
      { 8, "x,0,1/4"},
      { 8, "1/4,1/4,1/4"},
      { 4, "1/2,0,0"},
      { 4, "0,0,0"},
      { 4, "1/2,0,1/4"},
      { 4, "0,0,1/4"}
      };
      static const RawWyckoffPosition W73[] = {
      { 8, "0,1/4,z"},
      { 8, "1/4,y,0"},
      { 8, "x,0,1/4"},
      { 8, "1/4,1/4,1/4"},
      { 8, "0,0,0"}
      };
      static const RawWyckoffPosition W74[] = {
      { 8, "x,1/4,z"},
      { 8, "0,y,z"},
      { 8, "1/4,y,1/4"},
      { 8, "x,0,0"},
      { 4, "0,1/4,z"},
      { 4, "1/4,1/4,-1/4"},
      { 4, "1/4,1/4,1/4"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W75[] = {
      { 2, "0,1/2,z"},
      { 1, "1/2,1/2,z"},
      { 1, "0,0,z"}
      };
      static const RawWyckoffPosition W77[] = {
      { 2, "0,1/2,z"},
      { 2, "1/2,1/2,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W79[] = {
      { 4, "0,1/2,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W81[] = {
      { 2, "0,1/2,z"},
      { 2, "1/2,1/2,z"},
      { 2, "0,0,z"},
      { 1, "1/2,1/2,1/2"},
      { 1, "1/2,1/2,0"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W82[] = {
      { 4, "0,1/2,z"},
      { 4, "0,0,z"},
      { 2, "0,1/2,-1/4"},
      { 2, "0,1/2,1/4"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W83[] = {
      { 4, "x,y,1/2"},
      { 4, "x,y,0"},
      { 4, "0,1/2,z"},
      { 2, "1/2,1/2,z"},
      { 2, "0,0,z"},
      { 2, "0,1/2,1/2"},
      { 2, "0,1/2,0"},
      { 1, "1/2,1/2,1/2"},
      { 1, "1/2,1/2,0"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W84[] = {
      { 4, "x,y,0"},
      { 4, "0,1/2,z"},
      { 4, "1/2,1/2,z"},
      { 4, "0,0,z"},
      { 2, "1/2,1/2,1/4"},
      { 2, "0,0,1/4"},
      { 2, "0,1/2,1/2"},
      { 2, "0,1/2,0"},
      { 2, "1/2,1/2,0"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W85[] = {
      { 4, "1/4,-1/4,z"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"},
      { 2, "1/4,1/4,z"},
      { 2, "1/4,-1/4,1/2"},
      { 2, "1/4,-1/4,0"}
      };
      static const RawWyckoffPosition W86[] = {
      { 4, "1/4,1/4,z"},
      { 4, "-1/4,1/4,z"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"},
      { 2, "1/4,1/4,-1/4"},
      { 2, "1/4,1/4,1/4"}
      };
      static const RawWyckoffPosition W87[] = {
      { 8, "x,y,0"},
      { 8, "0,1/2,z"},
      { 8, "1/4,1/4,1/4"},
      { 4, "0,0,z"},
      { 4, "0,1/2,1/4"},
      { 4, "0,1/2,0"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W88[] = {
      { 8, "0,1/4,z"},
      { 8, "0,0,1/2"},
      { 8, "0,0,0"},
      { 4, "0,1/4,-3/8"},
      { 4, "0,1/4,1/8"}
      };
      static const RawWyckoffPosition W89[] = {
      { 4, "x,1/2,0"},
      { 4, "x,0,1/2"},
      { 4, "x,1/2,1/2"},
      { 4, "x,0,0"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,1/2"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,0"},
      { 4, "0,1/2,z"},
      { 2, "1/2,1/2,z"},
      { 2, "0,0,z"},
      { 2, "1/2,0,1/2"},
      { 2, "1/2,0,0"},
      { 1, "1/2,1/2,1/2"},
      { 1, "1/2,1/2,0"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W90[] = {
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,1/2"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,0"},
      { 4, "0,0,z"},
      { 2, "0,1/2,z"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W91[] = {
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,3/8"},
      { 4, "1/2,y,0"},
      { 4, "0,y,0"}
      };
      static const RawWyckoffPosition W92[] = {
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,0"}
      };
      static const RawWyckoffPosition W93[] = {
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,-1/4"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,1/4"},
      { 4, "x,1/2,0"},
      { 4, "x,0,1/2"},
      { 4, "x,1/2,1/2"},
      { 4, "x,0,0"},
      { 4, "0,1/2,z"},
      { 4, "1/2,1/2,z"},
      { 4, "0,0,z"},
      { 2, "1/2,1/2,1/4"},
      { 2, "0,0,1/4"},
      { 2, "0,1/2,1/2"},
      { 2, "0,1/2,0"},
      { 2, "1/2,1/2,0"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W94[] = {
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,1/2"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,0"},
      { 4, "0,1/2,z"},
      { 4, "0,0,z"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W95[] = {
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,-3/8"},
      { 4, "1/2,y,0"},
      { 4, "0,y,0"}
      };
      static const RawWyckoffPosition W97[] = {
      { 8, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,1/4"},
      { 8, "x,0,1/2"},
      { 8, "x,0,0"},
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,0"},
      { 8, "0,1/2,z"},
      { 4, "0,0,z"},
      { 4, "0,1/2,1/4"},
      { 4, "0,1/2,0"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W98[] = {
      { 8, "x,1/4,1/8"},
      { 8, "1/2*x-1/2*y,-1/2*x+1/2*y,0"},
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,0"},
      { 8, "0,0,z"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W99[] = {
      { 4, "x,1/2,z"},
      { 4, "x,0,z"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      { 2, "1/2,0,z"},
      { 1, "1/2,1/2,z"},
      { 1, "0,0,z"}
      };
      static const RawWyckoffPosition W100[] = {
      { 4, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,z"},
      { 2, "1/2,0,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W101[] = {
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      { 4, "0,1/2,z"},
      { 2, "1/2,1/2,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W102[] = {
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      { 4, "0,1/2,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W103[] = {
      { 4, "0,1/2,z"},
      { 2, "1/2,1/2,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W105[] = {
      { 4, "x,1/2,z"},
      { 4, "x,0,z"},
      { 2, "0,1/2,z"},
      { 2, "1/2,1/2,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W107[] = {
      { 8, "x,0,z"},
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      { 4, "0,1/2,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W108[] = {
      { 8, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,z"},
      { 4, "1/2,0,z"},
      { 4, "0,0,z"}
      };
      static const RawWyckoffPosition W109[] = {
      { 8, "0,y,z"},
      { 4, "0,0,z"}
      };
      static const RawWyckoffPosition W111[] = {
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      { 4, "0,1/2,z"},
      { 4, "x,1/2,0"},
      { 4, "x,0,1/2"},
      { 4, "x,1/2,1/2"},
      { 4, "x,0,0"},
      { 2, "1/2,1/2,z"},
      { 2, "0,0,z"},
      { 2, "1/2,0,1/2"},
      { 2, "1/2,0,0"},
      { 1, "1/2,1/2,0"},
      { 1, "0,0,1/2"},
      { 1, "1/2,1/2,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W112[] = {
      { 4, "0,1/2,z"},
      { 4, "1/2,1/2,z"},
      { 4, "0,0,z"},
      { 4, "0,y,1/4"},
      { 4, "x,1/2,1/4"},
      { 4, "1/2,y,1/4"},
      { 4, "x,0,1/4"},
      { 2, "1/2,1/2,0"},
      { 2, "0,0,0"},
      { 2, "0,1/2,1/4"},
      { 2, "1/2,1/2,1/4"},
      { 2, "1/2,0,1/4"},
      { 2, "0,0,1/4"}
      };
      static const RawWyckoffPosition W113[] = {
      { 4, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,z"},
      { 4, "0,0,z"},
      { 2, "0,1/2,z"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W114[] = {
      { 4, "0,1/2,z"},
      { 4, "0,0,z"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W115[] = {
      { 4, "x,1/2,z"},
      { 4, "x,0,z"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,1/2"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,0"},
      { 2, "0,1/2,z"},
      { 2, "1/2,1/2,z"},
      { 2, "0,0,z"},
      { 1, "0,0,1/2"},
      { 1, "1/2,1/2,1/2"},
      { 1, "1/2,1/2,0"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W116[] = {
      { 4, "0,1/2,z"},
      { 4, "1/2,1/2,z"},
      { 4, "0,0,z"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,-1/4"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,1/4"},
      { 2, "1/2,1/2,0"},
      { 2, "0,0,0"},
      { 2, "1/2,1/2,1/4"},
      { 2, "0,0,1/4"}
      };
      static const RawWyckoffPosition W117[] = {
      { 4, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,1/2"},
      { 4, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,0"},
      { 4, "0,1/2,z"},
      { 4, "0,0,z"},
      { 2, "0,1/2,1/2"},
      { 2, "0,1/2,0"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W118[] = {
      { 4, "0,1/2,z"},
      { 4, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,1/4"},
      { 4, "1/2*x-1/2*y+1/4,-1/2*x+1/2*y+1/4,1/4"},
      { 4, "0,0,z"},
      { 2, "0,1/2,-1/4"},
      { 2, "0,1/2,1/4"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W119[] = {
      { 8, "x,0,z"},
      { 8, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,1/4"},
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,0"},
      { 4, "0,1/2,z"},
      { 4, "0,0,z"},
      { 2, "0,1/2,-1/4"},
      { 2, "0,1/2,1/4"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W120[] = {
      { 8, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,0"},
      { 8, "0,1/2,z"},
      { 8, "0,0,z"},
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,1/4"},
      { 4, "0,1/2,0"},
      { 4, "0,1/2,1/4"},
      { 4, "0,0,0"},
      { 4, "0,0,1/4"}
      };
      static const RawWyckoffPosition W121[] = {
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      { 8, "0,1/2,z"},
      { 8, "x,0,1/2"},
      { 8, "x,0,0"},
      { 4, "0,0,z"},
      { 4, "0,1/2,1/4"},
      { 4, "0,1/2,0"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W122[] = {
      { 8, "x,1/4,1/8"},
      { 8, "0,0,z"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W123[] = {
      { 8, "x,1/2,z"},
      { 8, "x,0,z"},
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      { 8, "x,y,1/2"},
      { 8, "x,y,0"},
      { 4, "x,1/2,1/2"},
      { 4, "x,1/2,0"},
      { 4, "x,0,1/2"},
      { 4, "x,0,0"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,1/2"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,0"},
      { 4, "0,1/2,z"},
      { 2, "1/2,1/2,z"},
      { 2, "0,0,z"},
      { 2, "0,1/2,0"},
      { 2, "0,1/2,1/2"},
      { 1, "1/2,1/2,1/2"},
      { 1, "1/2,1/2,0"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W124[] = {
      { 8, "x,y,0"},
      { 8, "x,1/2,1/4"},
      { 8, "x,0,1/4"},
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,1/4"},
      { 8, "0,1/2,z"},
      { 4, "1/2,1/2,z"},
      { 4, "0,0,z"},
      { 4, "0,1/2,1/4"},
      { 4, "0,1/2,0"},
      { 2, "1/2,1/2,0"},
      { 2, "1/2,1/2,1/4"},
      { 2, "0,0,0"},
      { 2, "0,0,1/4"}
      };
      static const RawWyckoffPosition W125[] = {
      { 8, "1/2*x-1/2*y,-1/2*x+1/2*y,z"},
      { 8, "x,1/4,1/2"},
      { 8, "x,1/4,0"},
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,1/2"},
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,0"},
      { 4, "-1/4,1/4,z"},
      { 4, "1/4,1/4,z"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"},
      { 2, "-1/4,1/4,1/2"},
      { 2, "-1/4,1/4,0"},
      { 2, "1/4,1/4,1/2"},
      { 2, "1/4,1/4,0"}
      };
      static const RawWyckoffPosition W126[] = {
      { 8, "x,-1/4,1/4"},
      { 8, "x,1/4,1/4"},
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,1/4"},
      { 8, "1/4,-1/4,z"},
      { 8, "0,0,0"},
      { 4, "1/4,1/4,z"},
      { 4, "1/4,-1/4,0"},
      { 4, "1/4,-1/4,-1/4"},
      { 2, "1/4,1/4,-1/4"},
      { 2, "1/4,1/4,1/4"}
      };
      static const RawWyckoffPosition W127[] = {
      { 8, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,z"},
      { 8, "x,y,1/2"},
      { 8, "x,y,0"},
      { 4, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,1/2"},
      { 4, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,0"},
      { 4, "0,1/2,z"},
      { 4, "0,0,z"},
      { 2, "0,1/2,0"},
      { 2, "0,1/2,1/2"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W128[] = {
      { 8, "x,y,0"},
      { 8, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,1/4"},
      { 8, "0,1/2,z"},
      { 4, "0,0,z"},
      { 4, "0,1/2,1/4"},
      { 4, "0,1/2,0"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W129[] = {
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      { 8, "1/4,y,z"},
      { 8, "1/2*x-1/2*y,-1/2*x+1/2*y,1/2"},
      { 8, "1/2*x-1/2*y,-1/2*x+1/2*y,0"},
      { 4, "-1/4,1/4,z"},
      { 4, "0,0,1/2"},
      { 4, "0,0,0"},
      { 2, "1/4,1/4,z"},
      { 2, "-1/4,1/4,1/2"},
      { 2, "-1/4,1/4,0"}
      };
      static const RawWyckoffPosition W130[] = {
      { 8, "1/2*x-1/2*y,-1/2*x+1/2*y,1/4"},
      { 8, "-1/4,1/4,z"},
      { 8, "0,0,0"},
      { 4, "1/4,1/4,z"},
      { 4, "-1/4,1/4,0"},
      { 4, "-1/4,1/4,1/4"}
      };
      static const RawWyckoffPosition W131[] = {
      { 8, "x,y,0"},
      { 8, "1/2,y,z"},
      { 8, "0,y,z"},
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,1/4"},
      { 4, "x,1/2,0"},
      { 4, "x,0,1/2"},
      { 4, "x,1/2,1/2"},
      { 4, "x,0,0"},
      { 4, "0,1/2,z"},
      { 4, "1/2,1/2,z"},
      { 4, "0,0,z"},
      { 2, "1/2,1/2,1/4"},
      { 2, "0,0,1/4"},
      { 2, "0,1/2,1/2"},
      { 2, "0,1/2,0"},
      { 2, "1/2,1/2,0"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W132[] = {
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      { 8, "x,y,0"},
      { 8, "x,1/2,1/4"},
      { 8, "x,0,1/4"},
      { 8, "0,1/2,z"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,1/2"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,0"},
      { 4, "1/2,1/2,z"},
      { 4, "0,0,z"},
      { 4, "0,1/2,0"},
      { 4, "0,1/2,1/4"},
      { 2, "1/2,1/2,1/4"},
      { 2, "1/2,1/2,0"},
      { 2, "0,0,1/4"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W133[] = {
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,1/4"},
      { 8, "x,1/4,1/2"},
      { 8, "x,1/4,0"},
      { 8, "-1/4,1/4,z"},
      { 8, "1/4,1/4,z"},
      { 8, "0,0,0"},
      { 4, "-1/4,1/4,-1/4"},
      { 4, "1/4,1/4,1/4"},
      { 4, "-1/4,1/4,0"},
      { 4, "1/4,1/4,0"}
      };
      static const RawWyckoffPosition W134[] = {
      { 8, "1/2*x-1/2*y,-1/2*x+1/2*y,z"},
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,1/2"},
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,0"},
      { 8, "x,1/4,1/4"},
      { 8, "x,1/4,-1/4"},
      { 8, "1/4,1/4,z"},
      { 4, "-1/4,1/4,z"},
      { 4, "0,0,0"},
      { 4, "0,0,1/2"},
      { 4, "1/4,1/4,0"},
      { 4, "1/4,1/4,1/4"},
      { 2, "-1/4,1/4,1/4"},
      { 2, "1/4,-1/4,1/4"}
      };
      static const RawWyckoffPosition W135[] = {
      { 8, "x,y,0"},
      { 8, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,1/4"},
      { 8, "0,1/2,z"},
      { 8, "0,0,z"},
      { 4, "0,1/2,1/4"},
      { 4, "0,1/2,0"},
      { 4, "0,0,1/4"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W136[] = {
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      { 8, "x,y,0"},
      { 8, "0,1/2,z"},
      { 4, "1/2*x-1/2*y,-1/2*x+1/2*y,0"},
      { 4, "1/2*x+1/2*y,1/2*x+1/2*y,0"},
      { 4, "0,0,z"},
      { 4, "0,1/2,1/4"},
      { 4, "0,1/2,0"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W137[] = {
      { 8, "1/4,y,z"},
      { 8, "1/2*x-1/2*y,-1/2*x+1/2*y,1/4"},
      { 8, "0,0,0"},
      { 4, "1/4,1/4,z"},
      { 4, "-1/4,1/4,z"},
      { 2, "-1/4,1/4,1/4"},
      { 2, "-1/4,1/4,-1/4"}
      };
      static const RawWyckoffPosition W138[] = {
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      { 8, "1/2*x-1/2*y,-1/2*x+1/2*y,0"},
      { 8, "1/2*x-1/2*y,-1/2*x+1/2*y,1/2"},
      { 8, "-1/4,1/4,z"},
      { 4, "1/4,1/4,z"},
      { 4, "0,0,0"},
      { 4, "0,0,1/2"},
      { 4, "-1/4,1/4,-1/4"},
      { 4, "-1/4,1/4,0"}
      };
      static const RawWyckoffPosition W139[] = {
      {16, "0,y,z"},
      {16, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      {16, "x,y,0"},
      {16, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,1/4"},
      { 8, "x,1/2,0"},
      { 8, "x,0,0"},
      { 8, "1/2*x+1/2*y,1/2*x+1/2*y,0"},
      { 8, "0,1/2,z"},
      { 8, "1/4,1/4,1/4"},
      { 4, "0,0,z"},
      { 4, "0,1/2,1/4"},
      { 4, "0,1/2,0"},
      { 2, "0,0,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W140[] = {
      {16, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,z"},
      {16, "x,y,0"},
      {16, "x,0,1/4"},
      {16, "1/2*x+1/2*y,1/2*x+1/2*y,1/4"},
      { 8, "1/2*x+1/2*y-1/4,1/2*x+1/2*y+1/4,0"},
      { 8, "0,1/2,z"},
      { 8, "0,0,z"},
      { 8, "1/4,1/4,1/4"},
      { 4, "0,1/2,0"},
      { 4, "0,0,0"},
      { 4, "0,1/2,1/4"},
      { 4, "0,0,1/4"}
      };
      static const RawWyckoffPosition W141[] = {
      {16, "0,y,z"},
      {16, "1/2*x+1/2*y-1/8,1/2*x+1/2*y+1/8,-1/8"},
      {16, "x,0,0"},
      { 8, "0,1/4,z"},
      { 8, "0,0,1/2"},
      { 8, "0,0,0"},
      { 4, "0,1/4,3/8"},
      { 4, "0,-1/4,1/8"}
      };
      static const RawWyckoffPosition W142[] = {
      {16, "1/2*x+1/2*y-1/8,1/2*x+1/2*y+1/8,1/8"},
      {16, "x,0,1/4"},
      {16, "0,1/4,z"},
      {16, "0,0,0"},
      { 8, "0,1/4,1/8"},
      { 8, "0,1/4,3/8"}
      };
      static const RawWyckoffPosition W143[] = {
      { 1, "-1/3,1/3,z"},
      { 1, "1/3,-1/3,z"},
      { 1, "0,0,z"}
      };
      static const RawWyckoffPosition W146[] = {
      { 3, "0,0,z"}
      };
      static const RawWyckoffPosition W147[] = {
      { 3, "1/2,0,1/2"},
      { 3, "1/2,0,0"},
      { 2, "1/3,-1/3,z"},
      { 2, "0,0,z"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W148[] = {
      { 9, "1/2,0,0"},
      { 9, "1/2,0,1/2"},
      { 6, "0,0,z"},
      { 3, "0,0,1/2"},
      { 3, "0,0,0"}
      };
      static const RawWyckoffPosition W149[] = {
      { 3, "1/2*x-1/2*y,-1/2*x+1/2*y,1/2"},
      { 3, "1/2*x-1/2*y,-1/2*x+1/2*y,0"},
      { 2, "-1/3,1/3,z"},
      { 2, "1/3,-1/3,z"},
      { 2, "0,0,z"},
      { 1, "-1/3,1/3,1/2"},
      { 1, "-1/3,1/3,0"},
      { 1, "1/3,-1/3,1/2"},
      { 1, "1/3,-1/3,0"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W150[] = {
      { 3, "x-1/2*y,0,1/2"},
      { 3, "x-1/2*y,0,0"},
      { 2, "1/3,-1/3,z"},
      { 2, "0,0,z"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W151[] = {
      { 3, "1/2*x-1/2*y,-1/2*x+1/2*y,-1/6"},
      { 3, "1/2*x-1/2*y,-1/2*x+1/2*y,1/3"}
      };
      static const RawWyckoffPosition W152[] = {
      { 3, "x-1/2*y,0,-1/6"},
      { 3, "x-1/2*y,0,1/3"}
      };
      static const RawWyckoffPosition W153[] = {
      { 3, "1/2*x-1/2*y,-1/2*x+1/2*y,1/6"},
      { 3, "1/2*x-1/2*y,-1/2*x+1/2*y,-1/3"}
      };
      static const RawWyckoffPosition W154[] = {
      { 3, "x-1/2*y,0,1/6"},
      { 3, "x-1/2*y,0,-1/3"}
      };
      static const RawWyckoffPosition W155[] = {
      { 9, "x-1/2*y,0,1/2"},
      { 9, "x-1/2*y,0,0"},
      { 6, "0,0,z"},
      { 3, "0,0,1/2"},
      { 3, "0,0,0"}
      };
      static const RawWyckoffPosition W156[] = {
      { 3, "1/2*x-1/2*y,-1/2*x+1/2*y,z"},
      { 1, "-1/3,1/3,z"},
      { 1, "1/3,-1/3,z"},
      { 1, "0,0,z"}
      };
      static const RawWyckoffPosition W157[] = {
      { 3, "x-1/2*y,0,z"},
      { 2, "1/3,-1/3,z"},
      { 1, "0,0,z"}
      };
      static const RawWyckoffPosition W158[] = {
      { 2, "-1/3,1/3,z"},
      { 2, "1/3,-1/3,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W159[] = {
      { 2, "1/3,-1/3,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W160[] = {
      { 9, "1/2*x-1/2*y,-1/2*x+1/2*y,z"},
      { 3, "0,0,z"}
      };
      static const RawWyckoffPosition W161[] = {
      { 6, "0,0,z"}
      };
      static const RawWyckoffPosition W162[] = {
      { 6, "x-1/2*y,0,z"},
      { 6, "1/2*x-1/2*y,-1/2*x+1/2*y,1/2"},
      { 6, "1/2*x-1/2*y,-1/2*x+1/2*y,0"},
      { 4, "1/3,-1/3,z"},
      { 3, "1/2,0,1/2"},
      { 3, "1/2,0,0"},
      { 2, "0,0,z"},
      { 2, "1/3,-1/3,1/2"},
      { 2, "1/3,-1/3,0"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W163[] = {
      { 6, "1/2*x-1/2*y,-1/2*x+1/2*y,1/4"},
      { 6, "1/2,0,0"},
      { 4, "1/3,-1/3,z"},
      { 4, "0,0,z"},
      { 2, "-1/3,1/3,1/4"},
      { 2, "1/3,-1/3,1/4"},
      { 2, "0,0,0"},
      { 2, "0,0,1/4"}
      };
      static const RawWyckoffPosition W164[] = {
      { 6, "1/2*x-1/2*y,-1/2*x+1/2*y,z"},
      { 6, "x-1/2*y,0,1/2"},
      { 6, "x-1/2*y,0,0"},
      { 3, "1/2,0,1/2"},
      { 3, "1/2,0,0"},
      { 2, "1/3,-1/3,z"},
      { 2, "0,0,z"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W165[] = {
      { 6, "x-1/2*y,0,1/4"},
      { 6, "1/2,0,0"},
      { 4, "1/3,-1/3,z"},
      { 4, "0,0,z"},
      { 2, "0,0,0"},
      { 2, "0,0,1/4"}
      };
      static const RawWyckoffPosition W166[] = {
      {18, "1/2*x-1/2*y,-1/2*x+1/2*y,z"},
      {18, "x-1/2*y,0,1/2"},
      {18, "x-1/2*y,0,0"},
      { 9, "1/2,0,0"},
      { 9, "1/2,0,1/2"},
      { 6, "0,0,z"},
      { 3, "0,0,1/2"},
      { 3, "0,0,0"}
      };
      static const RawWyckoffPosition W167[] = {
      {18, "x-1/2*y,0,1/4"},
      {18, "1/2,0,0"},
      {12, "0,0,z"},
      { 6, "0,0,0"},
      { 6, "0,0,1/4"}
      };
      static const RawWyckoffPosition W168[] = {
      { 3, "1/2,0,z"},
      { 2, "1/3,-1/3,z"},
      { 1, "0,0,z"}
      };
      static const RawWyckoffPosition W171[] = {
      { 3, "1/2,1/2,z"},
      { 3, "0,0,z"}
      };
      static const RawWyckoffPosition W174[] = {
      { 3, "x,y,1/2"},
      { 3, "x,y,0"},
      { 2, "-1/3,1/3,z"},
      { 2, "1/3,-1/3,z"},
      { 2, "0,0,z"},
      { 1, "-1/3,1/3,1/2"},
      { 1, "-1/3,1/3,0"},
      { 1, "1/3,-1/3,1/2"},
      { 1, "1/3,-1/3,0"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W175[] = {
      { 6, "x,y,1/2"},
      { 6, "x,y,0"},
      { 6, "1/2,0,z"},
      { 4, "1/3,-1/3,z"},
      { 3, "1/2,0,1/2"},
      { 3, "1/2,0,0"},
      { 2, "0,0,z"},
      { 2, "1/3,-1/3,1/2"},
      { 2, "1/3,-1/3,0"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W176[] = {
      { 6, "x,y,1/4"},
      { 6, "1/2,0,0"},
      { 4, "1/3,-1/3,z"},
      { 4, "0,0,z"},
      { 2, "-1/3,1/3,1/4"},
      { 2, "1/3,-1/3,1/4"},
      { 2, "0,0,0"},
      { 2, "0,0,1/4"}
      };
      static const RawWyckoffPosition W177[] = {
      { 6, "1/2*x-1/2*y,-1/2*x+1/2*y,1/2"},
      { 6, "1/2*x-1/2*y,-1/2*x+1/2*y,0"},
      { 6, "x-1/2*y,0,1/2"},
      { 6, "x-1/2*y,0,0"},
      { 6, "1/2,0,z"},
      { 4, "1/3,-1/3,z"},
      { 3, "1/2,0,1/2"},
      { 3, "1/2,0,0"},
      { 2, "0,0,z"},
      { 2, "1/3,-1/3,1/2"},
      { 2, "1/3,-1/3,0"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W178[] = {
      { 6, "1/2*y,y,1/4"},
      { 6, "x-1/2*y,0,0"}
      };
      static const RawWyckoffPosition W179[] = {
      { 6, "1/2*y,y,-1/4"},
      { 6, "x-1/2*y,0,0"}
      };
      static const RawWyckoffPosition W180[] = {
      { 6, "1/2*y,y,1/2"},
      { 6, "1/2*y,y,0"},
      { 6, "x-1/2*y,0,1/2"},
      { 6, "x-1/2*y,0,0"},
      { 6, "1/2,0,z"},
      { 6, "0,0,z"},
      { 3, "1/2,0,1/2"},
      { 3, "1/2,0,0"},
      { 3, "0,0,1/2"},
      { 3, "0,0,0"}
      };
      static const RawWyckoffPosition W182[] = {
      { 6, "1/2*y,y,1/4"},
      { 6, "x-1/2*y,0,0"},
      { 4, "1/3,-1/3,z"},
      { 4, "0,0,z"},
      { 2, "1/3,-1/3,-1/4"},
      { 2, "1/3,-1/3,1/4"},
      { 2, "0,0,1/4"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W183[] = {
      { 6, "1/2*x-1/2*y,-1/2*x+1/2*y,z"},
      { 6, "x-1/2*y,0,z"},
      { 3, "1/2,0,z"},
      { 2, "1/3,-1/3,z"},
      { 1, "0,0,z"}
      };
      static const RawWyckoffPosition W184[] = {
      { 6, "1/2,0,z"},
      { 4, "1/3,-1/3,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W185[] = {
      { 6, "x-1/2*y,0,z"},
      { 4, "1/3,-1/3,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W186[] = {
      { 6, "1/2*x-1/2*y,-1/2*x+1/2*y,z"},
      { 2, "1/3,-1/3,z"},
      { 2, "0,0,z"}
      };
      static const RawWyckoffPosition W187[] = {
      { 6, "1/2*x-1/2*y,-1/2*x+1/2*y,z"},
      { 6, "x,y,1/2"},
      { 6, "x,y,0"},
      { 3, "1/2*x-1/2*y,-1/2*x+1/2*y,1/2"},
      { 3, "1/2*x-1/2*y,-1/2*x+1/2*y,0"},
      { 2, "-1/3,1/3,z"},
      { 2, "1/3,-1/3,z"},
      { 2, "0,0,z"},
      { 1, "-1/3,1/3,1/2"},
      { 1, "-1/3,1/3,0"},
      { 1, "1/3,-1/3,1/2"},
      { 1, "1/3,-1/3,0"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W188[] = {
      { 6, "x,y,1/4"},
      { 6, "1/2*x-1/2*y,-1/2*x+1/2*y,0"},
      { 4, "-1/3,1/3,z"},
      { 4, "1/3,-1/3,z"},
      { 4, "0,0,z"},
      { 2, "-1/3,1/3,1/4"},
      { 2, "-1/3,1/3,0"},
      { 2, "1/3,-1/3,1/4"},
      { 2, "1/3,-1/3,0"},
      { 2, "0,0,1/4"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W189[] = {
      { 6, "x,y,1/2"},
      { 6, "x,y,0"},
      { 6, "x-1/2*y,0,z"},
      { 4, "1/3,-1/3,z"},
      { 3, "x-1/2*y,0,1/2"},
      { 3, "x-1/2*y,0,0"},
      { 2, "0,0,z"},
      { 2, "1/3,-1/3,1/2"},
      { 2, "1/3,-1/3,0"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W190[] = {
      { 6, "x,y,1/4"},
      { 6, "x-1/2*y,0,0"},
      { 4, "1/3,-1/3,z"},
      { 4, "0,0,z"},
      { 2, "-1/3,1/3,1/4"},
      { 2, "1/3,-1/3,1/4"},
      { 2, "0,0,1/4"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W191[] = {
      {12, "x,y,1/2"},
      {12, "x,y,0"},
      {12, "1/2*y,y,z"},
      {12, "x-1/2*y,0,z"},
      { 6, "1/2*y,y,1/2"},
      { 6, "1/2*y,y,0"},
      { 6, "x-1/2*y,0,1/2"},
      { 6, "x-1/2*y,0,0"},
      { 6, "1/2,0,z"},
      { 4, "1/3,-1/3,z"},
      { 3, "1/2,0,1/2"},
      { 3, "1/2,0,0"},
      { 2, "0,0,z"},
      { 2, "1/3,-1/3,1/2"},
      { 2, "1/3,-1/3,0"},
      { 1, "0,0,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W192[] = {
      {12, "x,y,0"},
      {12, "1/2*y,y,1/4"},
      {12, "x-1/2*y,0,1/4"},
      {12, "1/2,0,z"},
      { 8, "1/3,-1/3,z"},
      { 6, "1/2,0,0"},
      { 6, "1/2,0,1/4"},
      { 4, "0,0,z"},
      { 4, "1/3,-1/3,0"},
      { 4, "1/3,-1/3,1/4"},
      { 2, "0,0,0"},
      { 2, "0,0,1/4"}
      };
      static const RawWyckoffPosition W193[] = {
      {12, "x-1/2*y,0,z"},
      {12, "x,y,1/4"},
      {12, "1/2*y,y,0"},
      { 8, "1/3,-1/3,z"},
      { 6, "x-1/2*y,0,1/4"},
      { 6, "1/2,0,0"},
      { 4, "0,0,z"},
      { 4, "1/3,-1/3,0"},
      { 4, "1/3,-1/3,1/4"},
      { 2, "0,0,0"},
      { 2, "0,0,1/4"}
      };
      static const RawWyckoffPosition W194[] = {
      {12, "1/2*y,y,z"},
      {12, "x,y,1/4"},
      {12, "x-1/2*y,0,0"},
      { 6, "1/2*y,y,1/4"},
      { 6, "1/2,0,0"},
      { 4, "1/3,-1/3,z"},
      { 4, "0,0,z"},
      { 2, "1/3,-1/3,-1/4"},
      { 2, "1/3,-1/3,1/4"},
      { 2, "0,0,1/4"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W195[] = {
      { 6, "x,1/2,1/2"},
      { 6, "x,1/2,0"},
      { 6, "x,0,1/2"},
      { 6, "x,0,0"},
      { 4, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 3, "1/2,0,0"},
      { 3, "0,1/2,1/2"},
      { 1, "1/2,1/2,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W196[] = {
      {24, "x,1/4,1/4"},
      {24, "x,0,0"},
      {16, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 4, "-1/4,-1/4,-1/4"},
      { 4, "1/4,1/4,1/4"},
      { 4, "1/2,1/2,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W197[] = {
      {12, "x,1/2,0"},
      {12, "x,0,0"},
      { 8, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 6, "0,1/2,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W198[] = {
      { 4, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"}
      };
      static const RawWyckoffPosition W199[] = {
      {12, "x,0,1/4"},
      { 8, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"}
      };
      static const RawWyckoffPosition W200[] = {
      {12, "1/2,y,z"},
      {12, "0,y,z"},
      { 8, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 6, "x,1/2,1/2"},
      { 6, "x,1/2,0"},
      { 6, "x,0,1/2"},
      { 6, "x,0,0"},
      { 3, "1/2,0,0"},
      { 3, "0,1/2,1/2"},
      { 1, "1/2,1/2,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W201[] = {
      {12, "x,-1/4,1/4"},
      {12, "x,1/4,1/4"},
      { 8, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 6, "1/4,-1/4,-1/4"},
      { 4, "1/2,1/2,1/2"},
      { 4, "0,0,0"},
      { 2, "1/4,1/4,1/4"}
      };
      static const RawWyckoffPosition W202[] = {
      {48, "0,y,z"},
      {48, "x,1/4,1/4"},
      {32, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {24, "x,0,0"},
      {24, "0,1/4,1/4"},
      { 8, "1/4,1/4,1/4"},
      { 4, "1/2,1/2,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W203[] = {
      {48, "x,1/8,1/8"},
      {32, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {16, "1/2,1/2,1/2"},
      {16, "0,0,0"},
      { 8, "-3/8,-3/8,-3/8"},
      { 8, "1/8,1/8,1/8"}
      };
      static const RawWyckoffPosition W204[] = {
      {24, "0,y,z"},
      {16, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {12, "x,0,1/2"},
      {12, "x,0,0"},
      { 8, "1/4,1/4,1/4"},
      { 6, "0,1/2,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W205[] = {
      { 8, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 4, "1/2,1/2,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W206[] = {
      {24, "x,0,1/4"},
      {16, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 8, "1/4,1/4,1/4"},
      { 8, "0,0,0"}
      };
      static const RawWyckoffPosition W207[] = {
      {12, "1/2,1/2*y+1/2*z,1/2*y+1/2*z"},
      {12, "0,1/2*y+1/2*z,1/2*y+1/2*z"},
      {12, "x,1/2,0"},
      { 8, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 6, "x,1/2,1/2"},
      { 6, "x,0,0"},
      { 3, "1/2,0,0"},
      { 3, "0,1/2,1/2"},
      { 1, "1/2,1/2,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W208[] = {
      {12, "1/4,1/2*y+1/2*z-1/4,1/2*y+1/2*z+1/4"},
      {12, "1/4,1/2*y-1/2*z+1/4,-1/2*y+1/2*z+1/4"},
      {12, "x,1/2,0"},
      {12, "x,0,1/2"},
      {12, "x,0,0"},
      { 8, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 6, "1/4,1/2,0"},
      { 6, "1/4,0,1/2"},
      { 6, "0,1/2,1/2"},
      { 4, "-1/4,-1/4,-1/4"},
      { 4, "1/4,1/4,1/4"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W209[] = {
      {48, "x,1/4,1/4"},
      {48, "1/2,1/2*y+1/2*z,1/2*y+1/2*z"},
      {48, "0,1/2*y+1/2*z,1/2*y+1/2*z"},
      {32, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {24, "x,0,0"},
      {24, "0,1/4,1/4"},
      { 8, "1/4,1/4,1/4"},
      { 4, "1/2,1/2,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W210[] = {
      {48, "1/8,1/2*y-1/2*z+1/8,-1/2*y+1/2*z+1/8"},
      {48, "x,0,0"},
      {32, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {16, "-3/8,-3/8,-3/8"},
      {16, "1/8,1/8,1/8"},
      { 8, "1/2,1/2,1/2"},
      { 8, "0,0,0"}
      };
      static const RawWyckoffPosition W211[] = {
      {24, "1/4,1/2*y-1/2*z+1/4,-1/2*y+1/2*z+1/4"},
      {24, "0,1/2*y+1/2*z,1/2*y+1/2*z"},
      {24, "x,1/2,0"},
      {16, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {12, "x,0,0"},
      {12, "1/4,1/2,0"},
      { 8, "1/4,1/4,1/4"},
      { 6, "0,1/2,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W212[] = {
      {12, "1/8,1/2*y-1/2*z+1/8,-1/2*y+1/2*z+1/8"},
      { 8, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 4, "-3/8,-3/8,-3/8"},
      { 4, "1/8,1/8,1/8"}
      };
      static const RawWyckoffPosition W213[] = {
      {12, "1/8,1/2*y+1/2*z-1/8,1/2*y+1/2*z+1/8"},
      { 8, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 4, "-1/8,-1/8,-1/8"},
      { 4, "3/8,3/8,3/8"}
      };
      static const RawWyckoffPosition W214[] = {
      {24, "1/8,1/2*y-1/2*z+1/8,-1/2*y+1/2*z+1/8"},
      {24, "1/8,1/2*y+1/2*z-1/8,1/2*y+1/2*z+1/8"},
      {24, "x,0,1/4"},
      {16, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {12, "-3/8,0,1/4"},
      {12, "1/8,0,1/4"},
      { 8, "-1/8,-1/8,-1/8"},
      { 8, "1/8,1/8,1/8"}
      };
      static const RawWyckoffPosition W215[] = {
      {12, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      {12, "x,1/2,0"},
      { 6, "x,1/2,1/2"},
      { 6, "x,0,0"},
      { 4, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 3, "1/2,0,0"},
      { 3, "0,1/2,1/2"},
      { 1, "1/2,1/2,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W216[] = {
      {48, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      {24, "x,1/4,1/4"},
      {24, "x,0,0"},
      {16, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 4, "-1/4,-1/4,-1/4"},
      { 4, "1/4,1/4,1/4"},
      { 4, "1/2,1/2,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W217[] = {
      {24, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      {24, "x,1/2,0"},
      {12, "x,0,0"},
      {12, "1/4,1/2,0"},
      { 8, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 6, "0,1/2,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W218[] = {
      {12, "x,0,1/2"},
      {12, "x,1/2,0"},
      {12, "x,0,0"},
      { 8, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 6, "1/4,0,1/2"},
      { 6, "1/4,1/2,0"},
      { 6, "0,1/2,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W219[] = {
      {48, "x,1/4,1/4"},
      {48, "x,0,0"},
      {32, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {24, "1/4,0,0"},
      {24, "0,1/4,1/4"},
      { 8, "1/4,1/4,1/4"},
      { 8, "0,0,0"}
      };
      static const RawWyckoffPosition W220[] = {
      {24, "x,0,1/4"},
      {16, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {12, "-1/8,0,1/4"},
      {12, "3/8,0,1/4"}
      };
      static const RawWyckoffPosition W221[] = {
      {24, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      {24, "1/2,y,z"},
      {24, "0,y,z"},
      {12, "1/2,1/2*y+1/2*z,1/2*y+1/2*z"},
      {12, "0,1/2*y+1/2*z,1/2*y+1/2*z"},
      {12, "x,1/2,0"},
      { 8, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 6, "x,1/2,1/2"},
      { 6, "x,0,0"},
      { 3, "1/2,0,0"},
      { 3, "0,1/2,1/2"},
      { 1, "1/2,1/2,1/2"},
      { 1, "0,0,0"}
      };
      static const RawWyckoffPosition W222[] = {
      {24, "1/4,1/2*y+1/2*z,1/2*y+1/2*z"},
      {24, "x,-1/4,1/4"},
      {16, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {12, "x,1/4,1/4"},
      {12, "0,-1/4,1/4"},
      { 8, "0,0,0"},
      { 6, "-1/4,1/4,1/4"},
      { 2, "1/4,1/4,1/4"}
      };
      static const RawWyckoffPosition W223[] = {
      {24, "0,y,z"},
      {24, "1/4,1/2*y+1/2*z-1/4,1/2*y+1/2*z+1/4"},
      {16, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {12, "x,1/2,0"},
      {12, "x,0,1/2"},
      {12, "x,0,0"},
      { 8, "1/4,1/4,1/4"},
      { 6, "1/4,1/2,0"},
      { 6, "1/4,0,1/2"},
      { 6, "0,1/2,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W224[] = {
      {24, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      {24, "1/2,1/2*y+1/2*z-1/4,1/2*y+1/2*z+1/4"},
      {24, "1/2,1/2*y-1/2*z,-1/2*y+1/2*z"},
      {24, "x,1/4,-1/4"},
      {12, "x,1/4,1/4"},
      {12, "1/2,1/4,-1/4"},
      { 8, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      { 6, "1/4,-1/4,-1/4"},
      { 4, "1/2,1/2,1/2"},
      { 4, "0,0,0"},
      { 2, "1/4,1/4,1/4"}
      };
      static const RawWyckoffPosition W225[] = {
      {96, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      {96, "0,y,z"},
      {48, "1/2,1/2*y+1/2*z,1/2*y+1/2*z"},
      {48, "0,1/2*y+1/2*z,1/2*y+1/2*z"},
      {48, "x,1/4,1/4"},
      {32, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {24, "x,0,0"},
      {24, "0,1/4,1/4"},
      { 8, "1/4,1/4,1/4"},
      { 4, "1/2,1/2,1/2"},
      { 4, "0,0,0"}
      };
      static const RawWyckoffPosition W226[] = {
      {96, "0,y,z"},
      {96, "1/4,1/2*y+1/2*z,1/2*y+1/2*z"},
      {64, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {48, "x,1/4,1/4"},
      {48, "x,0,0"},
      {24, "0,1/4,1/4"},
      {24, "1/4,0,0"},
      { 8, "0,0,0"},
      { 8, "1/4,1/4,1/4"}
      };
      static const RawWyckoffPosition W227[] = {
      {96, "0,1/2*y-1/2*z,-1/2*y+1/2*z"},
      {96, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      {48, "x,1/8,1/8"},
      {32, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {16, "1/2,1/2,1/2"},
      {16, "0,0,0"},
      { 8, "3/8,3/8,3/8"},
      { 8, "1/8,1/8,1/8"}
      };
      static const RawWyckoffPosition W228[] = {
      {96, "1/4,1/2*y-1/2*z,-1/2*y+1/2*z"},
      {96, "x,1/8,1/8"},
      {64, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {48, "-1/8,1/8,1/8"},
      {32, "0,0,0"},
      {32, "1/4,1/4,1/4"},
      {16, "1/8,1/8,1/8"}
      };
      static const RawWyckoffPosition W229[] = {
      {48, "1/2*x+1/2*y,1/2*x+1/2*y,z"},
      {48, "0,y,z"},
      {48, "1/4,1/2*y-1/2*z+1/4,-1/2*y+1/2*z+1/4"},
      {24, "0,1/2*y+1/2*z,1/2*y+1/2*z"},
      {24, "x,0,1/2"},
      {16, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {12, "x,0,0"},
      {12, "1/4,0,1/2"},
      { 8, "1/4,1/4,1/4"},
      { 6, "0,1/2,1/2"},
      { 2, "0,0,0"}
      };
      static const RawWyckoffPosition W230[] = {
      {48, "1/8,1/2*y-1/2*z+1/8,-1/2*y+1/2*z+1/8"},
      {48, "x,0,1/4"},
      {32, "1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z,1/3*x+1/3*y+1/3*z"},
      {24, "3/8,0,1/4"},
      {24, "1/8,0,1/4"},
      {16, "1/8,1/8,1/8"},
      {16, "0,0,0"}
      };
// END_COMPILED_IN_REFERENCE_DATA

      static const RawWyckoffTable RawWyckoffTables[] = {
      {  0, 0    },
      {  0, 0    }, //   1  P 1
      {  8, W2   }, //   2  P -1
      {  4, W3   }, //   3  P 1 2 1
      {  0, 0    }, //   4  P 1 21 1
      {  2, W5   }, //   5  C 1 2 1
      {  2, W6   }, //   6  P 1 m 1
      {  0, 0    }, //   7  P 1 c 1
      {  1, W8   }, //   8  C 1 m 1
      {  0, 0    }, //   9  C 1 c 1
      { 14, W10  }, //  10  P 1 2/m 1
      {  5, W11  }, //  11  P 1 21/m 1
      {  9, W12  }, //  12  C 1 2/m 1
      {  6, W13  }, //  13  P 1 2/c 1
      {  4, W14  }, //  14  P 1 21/c 1
      {  5, W15  }, //  15  C 1 2/c 1
      { 20, W16  }, //  16  P 2 2 2
      {  4, W17  }, //  17  P 2 2 21
      {  2, W18  }, //  18  P 21 21 2
      {  0, 0    }, //  19  P 21 21 21
      {  2, W20  }, //  20  C 2 2 21
      { 11, W21  }, //  21  C 2 2 2
      { 10, W22  }, //  22  F 2 2 2
      { 10, W23  }, //  23  I 2 2 2
      {  3, W24  }, //  24  I 21 21 21
      {  8, W25  }, //  25  P m m 2
      {  2, W26  }, //  26  P m c 21
      {  4, W27  }, //  27  P c c 2
      {  3, W28  }, //  28  P m a 2
      {  0, 0    }, //  29  P c a 21
      {  2, W30  }, //  30  P n c 2
      {  1, W31  }, //  31  P m n 21
      {  2, W18  }, //  32  P b a 2
      {  0, 0    }, //  33  P n a 21
      {  2, W18  }, //  34  P n n 2
      {  5, W35  }, //  35  C m m 2
      {  1, W36  }, //  36  C m c 21
      {  3, W37  }, //  37  C c c 2
      {  5, W38  }, //  38  A m m 2
      {  3, W39  }, //  39  A b m 2
      {  2, W40  }, //  40  A m a 2
      {  1, W41  }, //  41  A b a 2
      {  4, W42  }, //  42  F m m 2
      {  1, W43  }, //  43  F d d 2
      {  4, W44  }, //  44  I m m 2
      {  2, W45  }, //  45  I b a 2
      {  2, W40  }, //  46  I m a 2
      { 26, W47  }, //  47  P m m m
      { 12, W48  }, //  48  P n n n
      { 17, W49  }, //  49  P c c m
      { 12, W50  }, //  50  P b a n
      { 11, W51  }, //  51  P m m a
      {  4, W52  }, //  52  P n n a
      {  8, W53  }, //  53  P m n a
      {  5, W54  }, //  54  P c c a
      {  8, W55  }, //  55  P b a m
      {  4, W56  }, //  56  P c c n
      {  4, W57  }, //  57  P b c m
      {  7, W58  }, //  58  P n n m
      {  6, W59  }, //  59  P m m n
      {  3, W60  }, //  60  P b c n
      {  2, W61  }, //  61  P b c a
      {  3, W62  }, //  62  P n m a
      {  7, W63  }, //  63  C m c m
      {  6, W64  }, //  64  C m c a
      { 17, W65  }, //  65  C m m m
      { 12, W66  }, //  66  C c c m
      { 14, W67  }, //  67  C m m a
      {  8, W68  }, //  68  C c c a
      { 15, W69  }, //  69  F m m m
      {  7, W70  }, //  70  F d d d
      { 14, W71  }, //  71  I m m m
      { 10, W72  }, //  72  I b a m
      {  5, W73  }, //  73  I b c a
      {  9, W74  }, //  74  I m m a
      {  3, W75  }, //  75  P 4
      {  0, 0    }, //  76  P 41
      {  3, W77  }, //  77  P 42
      {  0, 0    }, //  78  P 43
      {  2, W79  }, //  79  I 4
      {  1, W41  }, //  80  I 41
      {  7, W81  }, //  81  P -4
      {  6, W82  }, //  82  I -4
      { 11, W83  }, //  83  P 4/m
      { 10, W84  }, //  84  P 42/m
      {  6, W85  }, //  85  P 4/n
      {  6, W86  }, //  86  P 42/n
      {  8, W87  }, //  87  I 4/m
      {  5, W88  }, //  88  I 41/a
      { 15, W89  }, //  89  P 4 2 2
      {  6, W90  }, //  90  P 4 21 2
      {  3, W91  }, //  91  P 41 2 2
      {  1, W92  }, //  92  P 41 21 2
      { 15, W93  }, //  93  P 42 2 2
      {  6, W94  }, //  94  P 42 21 2
      {  3, W95  }, //  95  P 43 2 2
      {  1, W92  }, //  96  P 43 21 2
      { 10, W97  }, //  97  I 4 2 2
      {  6, W98  }, //  98  I 41 2 2
      {  6, W99  }, //  99  P 4 m m
      {  3, W100 }, // 100  P 4 b m
      {  4, W101 }, // 101  P 42 c m
      {  3, W102 }, // 102  P 42 n m
      {  3, W103 }, // 103  P 4 c c
      {  2, W79  }, // 104  P 4 n c
      {  5, W105 }, // 105  P 42 m c
      {  2, W45  }, // 106  P 42 b c
      {  4, W107 }, // 107  I 4 m m
      {  3, W108 }, // 108  I 4 c m
      {  2, W109 }, // 109  I 41 m d
      {  1, W43  }, // 110  I 41 c d
      { 14, W111 }, // 111  P -4 2 m
      { 13, W112 }, // 112  P -4 2 c
      {  5, W113 }, // 113  P -4 21 m
      {  4, W114 }, // 114  P -4 21 c
      { 11, W115 }, // 115  P -4 m 2
      {  9, W116 }, // 116  P -4 c 2
      {  8, W117 }, // 117  P -4 b 2
      {  8, W118 }, // 118  P -4 n 2
      {  9, W119 }, // 119  I -4 m 2
      {  8, W120 }, // 120  I -4 c 2
      {  9, W121 }, // 121  I -4 2 m
      {  4, W122 }, // 122  I -4 2 d
      { 20, W123 }, // 123  P 4/m m m
      { 13, W124 }, // 124  P 4/m c c
      { 13, W125 }, // 125  P 4/n b m
      { 10, W126 }, // 126  P 4/n n c
      { 11, W127 }, // 127  P 4/m b m
      {  8, W128 }, // 128  P 4/m n c
      { 10, W129 }, // 129  P 4/n m m
      {  6, W130 }, // 130  P 4/n c c
      { 17, W131 }, // 131  P 42/m m c
      { 15, W132 }, // 132  P 42/m c m
      { 10, W133 }, // 133  P 42/n b c
      { 13, W134 }, // 134  P 42/n n m
      {  8, W135 }, // 135  P 42/m b c
      { 10, W136 }, // 136  P 42/m n m
      {  7, W137 }, // 137  P 42/n m c
      {  9, W138 }, // 138  P 42/n c m
      { 14, W139 }, // 139  I 4/m m m
      { 12, W140 }, // 140  I 4/m c m
      {  8, W141 }, // 141  I 41/a m d
      {  6, W142 }, // 142  I 41/a c d
      {  3, W143 }, // 143  P 3
      {  0, 0    }, // 144  P 31
      {  0, 0    }, // 145  P 32
      {  1, W146 }, // 146  R 3
      {  6, W147 }, // 147  P -3
      {  5, W148 }, // 148  R -3
      { 11, W149 }, // 149  P 3 1 2
      {  6, W150 }, // 150  P 3 2 1
      {  2, W151 }, // 151  P 31 1 2
      {  2, W152 }, // 152  P 31 2 1
      {  2, W153 }, // 153  P 32 1 2
      {  2, W154 }, // 154  P 32 2 1
      {  5, W155 }, // 155  R 3 2
      {  4, W156 }, // 156  P 3 m 1
      {  3, W157 }, // 157  P 3 1 m
      {  3, W158 }, // 158  P 3 c 1
      {  2, W159 }, // 159  P 3 1 c
      {  2, W160 }, // 160  R 3 m
      {  1, W161 }, // 161  R 3 c
      { 11, W162 }, // 162  P -3 1 m
      {  8, W163 }, // 163  P -3 1 c
      {  9, W164 }, // 164  P -3 m 1
      {  6, W165 }, // 165  P -3 c 1
      {  8, W166 }, // 166  R -3 m
      {  5, W167 }, // 167  R -3 c
      {  3, W168 }, // 168  P 6
      {  0, 0    }, // 169  P 61
      {  0, 0    }, // 170  P 65
      {  2, W171 }, // 171  P 62
      {  2, W171 }, // 172  P 64
      {  2, W159 }, // 173  P 63
      { 11, W174 }, // 174  P -6
      { 11, W175 }, // 175  P 6/m
      {  8, W176 }, // 176  P 63/m
      { 13, W177 }, // 177  P 6 2 2
      {  2, W178 }, // 178  P 61 2 2
      {  2, W179 }, // 179  P 65 2 2
      { 10, W180 }, // 180  P 62 2 2
      { 10, W180 }, // 181  P 64 2 2
      {  8, W182 }, // 182  P 63 2 2
      {  5, W183 }, // 183  P 6 m m
      {  3, W184 }, // 184  P 6 c c
      {  3, W185 }, // 185  P 63 c m
      {  3, W186 }, // 186  P 63 m c
      { 14, W187 }, // 187  P -6 m 2
      { 11, W188 }, // 188  P -6 c 2
      { 11, W189 }, // 189  P -6 2 m
      {  8, W190 }, // 190  P -6 2 c
      { 17, W191 }, // 191  P 6/m m m
      { 12, W192 }, // 192  P 6/m c c
      { 11, W193 }, // 193  P 63/m c m
      { 11, W194 }, // 194  P 63/m m c
      {  9, W195 }, // 195  P 2 3
      {  7, W196 }, // 196  F 2 3
      {  5, W197 }, // 197  I 2 3
      {  1, W198 }, // 198  P 21 3
      {  2, W199 }, // 199  I 21 3
      { 11, W200 }, // 200  P m -3
      {  7, W201 }, // 201  P n -3
      {  8, W202 }, // 202  F m -3
      {  6, W203 }, // 203  F d -3
      {  7, W204 }, // 204  I m -3
      {  3, W205 }, // 205  P a -3
      {  4, W206 }, // 206  I a -3
      { 10, W207 }, // 207  P 4 3 2
      { 12, W208 }, // 208  P 42 3 2
      {  9, W209 }, // 209  F 4 3 2
      {  7, W210 }, // 210  F 41 3 2
      {  9, W211 }, // 211  I 4 3 2
      {  4, W212 }, // 212  P 43 3 2
      {  4, W213 }, // 213  P 41 3 2
      {  8, W214 }, // 214  I 41 3 2
      {  9, W215 }, // 215  P -4 3 m
      {  8, W216 }, // 216  F -4 3 m
      {  7, W217 }, // 217  I -4 3 m
      {  8, W218 }, // 218  P -4 3 n
      {  7, W219 }, // 219  F -4 3 c
      {  4, W220 }, // 220  I -4 3 d
      { 13, W221 }, // 221  P m -3 m
      {  8, W222 }, // 222  P n -3 n
      { 11, W223 }, // 223  P m -3 n
      { 11, W224 }, // 224  P n -3 m
      { 11, W225 }, // 225  F m -3 m
      {  9, W226 }, // 226  F m -3 c
      {  8, W227 }, // 227  F d -3 m
      {  7, W228 }, // 228  F d -3 c
      { 11, W229 }, // 229  I m -3 m
      {  7, W230 }  // 230  I a -3 d
      };

      static const int GeneralPositionMultiplicity[] = {
        0,
        1, 2, 2, 2, 4, 2, 2, 4, 4, 4, 4, 8, 4, 4, 8, 4, 4, 4, 4, 8, 8,
        16, 8, 8, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 8, 8, 8, 8, 8, 8, 8, 16,
        16, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 16,
        16, 16, 16, 16, 16, 32, 32, 16, 16, 16, 16, 4, 4, 4, 4, 8, 8, 4,
        8, 8, 8, 8, 8, 16, 16, 8, 8, 8, 8, 8, 8, 8, 8, 16, 16, 8, 8, 8,
        8, 8, 8, 8, 8, 16, 16, 16, 16, 8, 8, 8, 8, 8, 8, 8, 8, 16, 16,
        16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
        16, 16, 32, 32, 32, 32, 3, 3, 3, 9, 6, 18, 6, 6, 6, 6, 6, 6, 18,
        6, 6, 6, 6, 18, 18, 12, 12, 12, 12, 36, 36, 6, 6, 6, 6, 6, 6, 6,
        12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
        24, 24, 24, 24, 12, 48, 24, 12, 24, 24, 24, 96, 96, 48, 24, 48,
        24, 24, 96, 96, 48, 24, 24, 48, 24, 96, 48, 24, 96, 48, 48, 48,
        48, 48, 192, 192, 192, 192, 96, 96
      };

    } // namespace ReferenceSettings
  } // namespace tables

  static const char WyckoffLetters[] = "abcdefghijklmnopqrstuvwxyz@";

  std::size_t WyckoffTable::LookupIndex(char Letter) const
  {
    for (std::size_t i=0; WyckoffLetters[i]; i++) {
      if (WyckoffLetters[i] == Letter) {
        if (i < N()) return N() - 1 - i;
        throw error("Wyckoff letter out of range.");
      }
    }
    throw error("Not a Wyckoff letter.");
  }

  void WyckoffTable::InitializeOperations(const SpaceGroupInfo& SgInfo)
  {
    using namespace tables::ReferenceSettings;
    const int& SgNumber = SgInfo.SgNumber();
    cctbx_assert(1 <= SgNumber && SgNumber <= 230);
    const RotMx& CBMxR = SgInfo.CBOp().M().Rpart();
    int BF = CBMxR.BF();
    boost::rational<int> FactorM(CBMxR.det(), BF * BF * BF);
    boost::rational<int> M = GeneralPositionMultiplicity[SgNumber] * FactorM;
    cctbx_assert(M.denominator() == 1);
    const RawWyckoffTable& RWTab = RawWyckoffTables[SgNumber];
    cctbx_assert(RWTab.N < 27);
    char Letter = WyckoffLetters[RWTab.N];
    m_Operations.push_back(WyckoffPosition(M.numerator(), Letter, RTMx(1, 1)));
    ChOfBasisOp InvCBOp = SgInfo.CBOp().swap();
    RTMx ReferenceSpecialOp;
    for(int i=0;i<RWTab.N;i++) {
      try {
        ReferenceSpecialOp = RTMx(RWTab.RWO[i].xyz, "", 6, 24);
      }
      catch (const error&) {
        throw cctbx_internal_error();
      }
      M = RWTab.RWO[i].M * FactorM;
      cctbx_assert(M.denominator() == 1);
      Letter = WyckoffLetters[RWTab.N - 1 - i];
      RTMx SpecialOp = InvCBOp.apply(ReferenceSpecialOp);
      m_Operations.push_back(WyckoffPosition(M.numerator(), Letter,SpecialOp));
    }
  }

  void WyckoffTable::expand(const SpaceGroup& SgOps) {
    for(std::size_t i=0;i<N();i++) {
      m_Operations[i].expand(SgOps);
    }
  }

  WyckoffTable::WyckoffTable(const SpaceGroupInfo& SgInfo, bool auto_expand) {
    InitializeOperations(SgInfo);
    if (auto_expand) expand(SgInfo.SgOps());
  }

  const WyckoffMapping
  WyckoffTable::getWyckoffMapping(const uctbx::UnitCell& uc,
                                  const SpaceGroup& SgOps,
                                  const fractional<double>& X,
                                  double SnapRadius) const
  {
    fractional<double> NormX = X.modShort();
    TrVec NormU = detail::getUnitShifts(NormX - X);
    for (std::size_t iWP = N() - 1; iWP > 0; iWP--) {
      WyckoffMapping result;
      double ShortestDistance2 = uc.getLongestVector2();
      for (int iOp = 0; iOp < SgOps.OrderZ(); iOp++) {
        RTMx SymOp = SgOps(iOp).modShort();
        fractional<double> SX = SymOp * NormX;
        TrVec U(1);
        for (U[0] = -1; U[0] <= 1; U[0]++)
        for (U[1] = -1; U[1] <= 1; U[1]++)
        for (U[2] = -1; U[2] <= 1; U[2]++) {
          fractional<double> USX = SX + U;
          fractional<double> SnappedUSX = m_Operations[iWP].SpecialOp() * USX;
          double Dist2 = uc.Distance2(SnappedUSX, USX);
          if (ShortestDistance2 > Dist2) {
            ShortestDistance2 = Dist2;
            result = WyckoffMapping(
              m_Operations[iWP],
              RTMx(SymOp.Rpart(),
                   SymOp.Tpart().plus(U).plus(SymOp.Rpart().multiply(NormU))));
          }
        }
      }
      if (ShortestDistance2 <= SnapRadius * SnapRadius) {
        return result;
      }
    }
    return WyckoffMapping(m_Operations[0], RTMx(1, 1));
  }

  namespace detail {

    TrVec SolveInZ(RotMx M, TrVec b)
    {
      int f = lcm(M.BF(), b.BF());
      M = M.scale(f / M.BF());
      b = b.scale(f / b.BF());
      RotMx P(1), Q(1);
      int nd = SmithNormalForm(M.elems, 3, 3, P.elems, Q.elems);
      cctbx_assert(nd >= 0 && nd <= 3);
      TrVec Pb = P * b;
      int i;
      for (i = nd; i < 3; i++) {
        if (Pb[i] != 0) return TrVec(0);
      }
      TrVec xp(1);
      for (i = 0; i < nd; i++) {
        int d = M[i * nd + i];
        if (Pb[i] % d != 0) return TrVec(0);
        xp[i] = Pb[i] / d;
      }
      return Q * xp;
    }

  } // namespace detail

  const WyckoffMapping
  WyckoffTable::getWyckoffMapping(const SiteSymmetry& SS) const
  {
    const WyckoffTable& WTab = *this;
    if (WTab[0].M() == SS.M()) return WyckoffMapping(WTab[0], RTMx(1, 1));
    const RotMx& Rs = SS.SpecialOp().Rpart();
    const TrVec& Ts = SS.SpecialOp().Tpart();
    const SpaceGroup& SgOps = SS.m_Parameters->m_SgOps;
    for(std::size_t iWP=1;iWP<WTab.N();iWP++) {
      if (WTab[iWP].M() == SS.M()) {
        const RotMx& Rw = WTab[iWP].SpecialOp().Rpart();
        const TrVec& Tw = WTab[iWP].SpecialOp().Tpart();
        for(int iSMx=0;iSMx<SgOps.OrderP();iSMx++) {
          RTMx SMx = SgOps(iSMx);
          const RotMx& R = SMx.Rpart();
          if (Rw.multiply(R) == R.multiply(Rs)) {
            for(int iLTr=0;iLTr<SgOps.nLTr();iLTr++) {
              TrVec T = SMx.Tpart() + SgOps.LTr(iLTr);
              RotMx RwmI = (Rw - Rw.Unit()).cancel();
              TrVec b = R.multiply(Ts).plus(T).minus(Rw.multiply(T)).minus(Tw);
              TrVec U = detail::SolveInZ(RwmI, b);
              if (U.isValid()) {
                return WyckoffMapping(WTab[iWP], RTMx(R.cancel(), T.plus(U)));
              }
            }
          }
        }
      }
    }
    throw error("Cannot determine mapping to Wyckoff position.");
  }

}} // namespace cctbx::sgtbx
